from enum import auto

from gw.enums import GwStrEnum


class TempCalcMethod(GwStrEnum):
    """
    What method is used to calculate temperature as a function of voltage?
    Values:
      - SimpleBetaForPico: Use the Beta method for an NTC thermistor, with the (flawed)
        assumption that the pico which the thermistor is attached to has a fixed resistance.
        Requires parameters PicoResistance and ThermistorBeta.
      - SimpleBeta: Use the Beta method for an NTC thermistor. Requires ThermistorBeta
        as a parameter.

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#tempcalcmethod)
    """

    SimpleBetaForPico = auto()
    SimpleBeta = auto()

    @classmethod
    def default(cls) -> "TempCalcMethod":
        return cls.SimpleBeta

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "temp.calc.method"

    @classmethod
    def enum_version(cls) -> str:
        return "000"



from gwproto.data_classes.components.ads111x_based_component import (
    Ads111xBasedComponent,
)
from gwproto.data_classes.components.component import Component
from gwproto.data_classes.components.dfr_component import DfrComponent
from gwproto.data_classes.components.electric_meter_component import (
    ElectricMeterComponent,
)
from gwproto.data_classes.components.fibaro_smart_implant_component import (
    FibaroSmartImplantComponent,
)
from gwproto.data_classes.components.hubitat_component import HubitatComponent
from gwproto.data_classes.components.hubitat_poller_component import (
    HubitatPollerComponent,
)
from gwproto.data_classes.components.hubitat_tank_component import HubitatTankComponent
from gwproto.data_classes.components.i2c_multichannel_dt_relay_component import (
    I2cMultichannelDtRelayComponent,
)
from gwproto.data_classes.components.pico_btu_meter_component import (
    PicoBtuMeterComponent,
)
from gwproto.data_classes.components.pico_flow_module_component import (
    PicoFlowModuleComponent,
)
from gwproto.data_classes.components.pico_tank_module_component import (
    PicoTankModuleComponent,
)
from gwproto.data_classes.components.resistive_heater_component import (
    ResistiveHeaterComponent,
)
from gwproto.data_classes.components.rest_poller_component import RESTPollerComponent
from gwproto.data_classes.components.web_server_component import WebServerComponent

__all__ = [
    "Ads111xBasedComponent",
    "Component",
    "DfrComponent",
    "ElectricMeterComponent",
    "FibaroSmartImplantComponent",
    "HubitatComponent",
    "HubitatPollerComponent",
    "HubitatTankComponent",
    "I2cMultichannelDtRelayComponent",
    "PicoBtuMeterComponent",
    "PicoFlowModuleComponent",
    "PicoTankModuleComponent",
    "RESTPollerComponent",
    "ResistiveHeaterComponent",
    "WebServerComponent",
]



"""ElectricMeterComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types.electric_meter_cac_gt import ElectricMeterCacGt
from gwproto.named_types.electric_meter_component_gt import ElectricMeterComponentGt


class ElectricMeterComponent(
    Component[ElectricMeterComponentGt, ElectricMeterCacGt]
): ...



"""PicoBtuMeterComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types import ComponentAttributeClassGt, PicoBtuMeterComponentGt


class PicoBtuMeterComponent(
    Component[PicoBtuMeterComponentGt, ComponentAttributeClassGt]
): ...



from typing import Literal

from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.hubitat_tank_gt import HubitatTankSettingsGt


class HubitatTankComponentGt(ComponentGt):
    Tank: HubitatTankSettingsGt
    TypeName: Literal["hubitat.tank.component.gt"] = "hubitat.tank.component.gt"



"""Type heartbeat.b, version 001"""

from typing import Literal

from pydantic import BaseModel, Field

from gwproto.property_format import HexChar, LeftRightDotStr, UTCMilliseconds, UUID4Str


class HeartbeatB(BaseModel):
    FromGNodeAlias: LeftRightDotStr
    FromGNodeInstanceId: UUID4Str
    MyHex: HexChar = Field(
        title="Hex character getting sent",
        default="0",
    )
    YourLastHex: HexChar = Field(
        title="Last hex character received from heartbeat partner.",
    )
    LastReceivedTimeUnixMs: UTCMilliseconds = Field(
        title="Time YourLastHex was received on my clock",
    )
    SendTimeUnixMs: UTCMilliseconds = Field(
        title="Time this message is made and sent on my clock",
    )
    StartingOver: bool = Field(
        title="True if the heartbeat initiator wants to start the volley over",
        description=(
            "(typically the AtomicTNode in an AtomicTNode / SCADA pair) wants to start the heartbeating "
            "volley over. The result is that its partner will not expect the initiator to know "
            "its last Hex."
        ),
    )
    TypeName: Literal["heartbeat.b"] = "heartbeat.b"
    Version: str = "001"

    def __hash__(self) -> int:
        return hash((type(self), *self.__dict__.values()))



"""Type i2c.multichannel.dt.relay.component.gt, version 000"""

from typing import Literal

from pydantic import ConfigDict, StrictInt, model_validator
from typing_extensions import Self

from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.relay_actor_config import RelayActorConfig


class I2cMultichannelDtRelayComponentGt(ComponentGt):
    I2cAddressList: list[StrictInt]
    ConfigList: list[RelayActorConfig]
    TypeName: Literal["i2c.multichannel.dt.relay.component.gt"] = (
        "i2c.multichannel.dt.relay.component.gt"
    )
    Version: str = "002"

    model_config = ConfigDict(extra="allow", use_enum_values=True)

    @model_validator(mode="after")
    def check_axiom_3(self) -> Self:
        """g
        Axiom 2: Actor and Idx Consistency.
        There are no duplicates of ActorName or RelayIdx in the RelayConfigList
        """
        # Implement Axiom(s)
        return self



from enum import auto

from gw.enums import GwStrEnum


class ThermistorDataMethod(GwStrEnum):
    """
    What method is used to go from raw voltage readings to captured temperature readings.
    Values:
      - SimpleBeta: Using the beta formula with a calibrated open voltage reading, transmitting
        raw polled data.
      - BetaWithExponentialAveraging: Using the beta formula with a calibrated open voltage
        reading, and then some sort of exponential weighted averaging on polled data.

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#thermistordatamethod)
    """

    SimpleBeta = auto()
    BetaWithExponentialAveraging = auto()

    @classmethod
    def default(cls) -> "ThermistorDataMethod":
        return cls.SimpleBeta

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "thermistor.data.method"

    @classmethod
    def enum_version(cls) -> str:
        return "000"



from enum import auto

from gw.enums import GwStrEnum


class ActorClass(GwStrEnum):
    """
    Determines the code running Spaceheat Nodes supervised by Spaceheat SCADA software
    Values:
      - NoActor: A SpaceheatNode that does not have any code running on its behalf within
        the SCADA, but is instead only a reference object (for example, a tank of hot water
        or a resistive element) that can be discussed (for example, the power drawn by the resistive
        element can be measured) or evaluated (for example, a set of 5 different temperatures
        in different places on the tank can be used to estimate total thermal energy in the
        tank).
      - Scada: The SCADA actor is the prime piece of code running and supervising other
        ProActors within the SCADA code. It is also responsible for managing the state of TalkingWith
        the AtomicTNode, as well maintaining and reporting a boolean state variable that indicates
        whether it is following dispatch commands from the AtomicTNode XOR following dispatch
        commands from its own HomeAlone actor.
      - HomeAlone: HomeAlone is an abstract Spaceheat Actor responsible for dispatching
        the SCADA when it is not talking with the AtomicTNode.
      - BooleanActuator: A SpaceheatNode representing a generic boolean actuator capable
        of turning on (closing a circuit) or turning off (opening a circuit). If the device
        is a relay that can be directly energized or de-energized, recommend using Relay actor
        instead.
      - PowerMeter: A SpaceheatNode representing the power meter that is used to settle
        financial transactions with the TerminalAsset. That is, this is the power meter whose
        accuracy is certified in the creation of the TerminalAsset GNode via creation of the
        TaDeed. [More Info](https://gridworks.readthedocs.io/en/latest/terminal-asset.html).
      - Atn: A SpaceheatNode representing the AtomicTNode. Note that the code running
        the AtomicTNode is not local within the SCADA code, except for a stub used for testing
        purposes. [More Info](https://gridworks.readthedocs.io/en/latest/atomic-t-node.html).
      - SimpleSensor: A SpaceheatNode representing a sensor that measures a single category
        of quantity (for example, temperature) for a single object (for example, on a pipe). [More Info](https://gridworks-protocol.readthedocs.io/en/latest/simple-sensor.html).
      - MultipurposeSensor: A sensor that either reads multiple kinds of readings from
        the same sensing device (for example reads current and voltage), reads multiple different
        objects (temperature from two different thermisters) or both. [More Info](https://gridworks-protocol.readthedocs.io/en/latest/multipurpose-sensor.html).
      - Thermostat: A SpaceheatNode representing a thermostat.
      - HubitatTelemetryReader: A generic actor for reading telemetry data from a Hubitat
        Home Automation Hub LAN API. [More Info](https://drive.google.com/drive/u/0/folders/1AqAU_lC2phzuI9XRYvogiIYA7GXNtlr6).
      - HubitatTankModule: The actor for running a GridWorks TankModule, comprised of
        two Z-Wave Fibaro temp sensors built together inside a small container that has 4 thermistors
        attached. These are designed to be installed from top (1) to bottom (4) on a stratified
        thermal storage tank. [More Info](https://drive.google.com/drive/u/0/folders/1GSxDd8Naf1GKK_fSOgQU933M1UcJ4r8q).
      - HubitatPoller: An actor for representing a somewhat generic ShNode (like a thermostat)
        that can be polled through the Hubitat.
      - I2cRelayMultiplexer: Responsible for maintaining a single i2c bus object
      - FlowTotalizer: Attached to a driver that reads liquid flow by counting pulses
        from a flow meter that creates pulses and integrating the result (known as a totalizer
        in the industry).
      - Relay: An actor representing a relay. If the device is indeed relay that can be
        directly energized or de-energized, recommend using Relay instead of BooleanActuator
      - Admin: Actor for taking control of all of the actuators - flattening the hierarchy
        and disabling all finite state machines.
      - Fsm: Actor Class for Finite State Machine actors. For these actors, the code is
        determined by the ShNode Name instead of just the ActorClass.
      - Parentless: An actor that has no parent and is also not the primary SCADA. Used
        when there are multiple devices in the SCADA's system. For example, two Pis - one running
        the primary SCADA code and temp sensors, the other running relays and 0-10V output devices.
        A Parentless actor on the second Pi is responsible for spinning up the relay- and 0-10V
        output actors.
      - Hubitat: An actor for representing a Hubitat for receiving Hubitat events over
        HTTP.
      - HoneywellThermostat: An actor for representing a Honeywell Hubitat thermostat
        which can load thermostat heating state change messages into status reports.
      - ApiTankModule
      - ApiFlowModule
      - PicoCycler
      - I2cDfrMultiplexer
      - ZeroTenOutputer
      - AtomicAlly: Direct report of Atn when the Scada is in Atn mode.
      - SynthGenerator
      - FakeAtn
      - PumpDoctor: An actor that monitors and resets pumps if necessary
      - DefrostManager: Actor that handles the defrost cycle of a heat pump.

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#shactorclass)
      - [More Info](https://gridworks-protocol.readthedocs.io/en/latest/actor-class.html)
    """

    NoActor = auto()
    Scada = auto()
    HomeAlone = auto()
    BooleanActuator = auto()
    PowerMeter = auto()
    Atn = auto()
    SimpleSensor = auto()
    MultipurposeSensor = auto()
    Thermostat = auto()
    HubitatTelemetryReader = auto()
    HubitatTankModule = auto()
    HubitatPoller = auto()
    I2cRelayMultiplexer = auto()
    FlowTotalizer = auto()
    Relay = auto()
    Admin = auto()
    Fsm = auto()
    Parentless = auto()
    Hubitat = auto()
    HoneywellThermostat = auto()
    ApiTankModule = auto()
    ApiFlowModule = auto()
    PicoCycler = auto()
    I2cDfrMultiplexer = auto()
    ZeroTenOutputer = auto()
    AtomicAlly = auto()
    SynthGenerator = auto()
    FakeAtn = auto()
    PumpDoctor = auto()
    StratBoss = auto()
    HpRelayBoss = auto()
    SiegLoop = auto()
    HpBoss = auto()
    ApiBtuMeter = auto()

    @classmethod
    def default(cls) -> "ActorClass":
        return cls.NoActor

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "sh.actor.class"

    @classmethod
    def enum_version(cls) -> str:
        return "008"



"""Type fsm.atomic.report, version 000"""

from typing import Literal, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, model_validator
from typing_extensions import Self

from gwproto.enums import FsmActionType, FsmReportType
from gwproto.property_format import (
    HandleName,
    LeftRightDotStr,
    UTCMilliseconds,
    UUID4Str,
)


class FsmAtomicReport(BaseModel):
    MachineHandle: HandleName
    StateEnum: str
    ReportType: FsmReportType
    ActionType: Optional[FsmActionType] = None
    Action: Optional[StrictInt] = None
    EventEnum: Optional[LeftRightDotStr] = None
    Event: Optional[str] = None
    FromState: Optional[str] = None
    ToState: Optional[str] = None
    UnixTimeMs: UTCMilliseconds
    TriggerId: UUID4Str
    TypeName: Literal["fsm.atomic.report"] = "fsm.atomic.report"
    Version: str = "000"

    model_config = ConfigDict(extra="allow")

    @model_validator(mode="after")
    def check_axiom_1(self) -> Self:
        """
        Axiom 1: Action and ActionType exist iff  ReportType is Action.
        The Optional Attributes ActionType and Action exist if and only if IsAction is true.
        """
        # Implement check for axiom 1"
        return self

    @model_validator(mode="after")
    def check_axiom_2(self) -> Self:
        """
        Axiom 2: If Action exists, then it belongs to the un-versioned enum selected in the ActionType.

        """
        # Implement check for axiom 2"
        return self

    @model_validator(mode="after")
    def check_axiom_3(self) -> Self:
        """
        Axiom 3: EventType, Event, FromState, ToState exist iff ReportType is Event.

        """
        # Implement check for axiom 3"
        return self



from typing import Optional

import yarl

from gwproto.data_classes.components.component import Component
from gwproto.named_types.component_attribute_class_gt import ComponentAttributeClassGt
from gwproto.named_types.hubitat_component_gt import HubitatComponentGt


class HubitatComponent(Component[HubitatComponentGt, ComponentAttributeClassGt]):
    web_listener_nodes: set[str]

    def __init__(self, gt: HubitatComponentGt, cac: ComponentAttributeClassGt) -> None:
        super().__init__(gt, cac)
        self.web_listener_nodes = set()

    def urls(self) -> dict[str, Optional[yarl.URL]]:
        return self.gt.urls()

    def add_web_listener(self, web_listener_node: str) -> None:
        self.web_listener_nodes.add(web_listener_node)



"""Type alert, version 000"""

from typing import Literal, Optional

from pydantic import BaseModel

from gwproto.property_format import (
    LeftRightDotStr,
    SpaceheatName,
    UTCSeconds,
)


class Alert(BaseModel):
    FromGNodeAlias: LeftRightDotStr
    AboutNode: Optional[SpaceheatName] = None
    UnixS: UTCSeconds
    Summary: str
    OpsGenieAlias: Optional[str] = None
    TypeName: Literal["alert"] = "alert"
    Version: str = "000"



"""Ads111xBasedComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types import Ads111xBasedCacGt, Ads111xBasedComponentGt


class Ads111xBasedComponent(Component[Ads111xBasedComponentGt, Ads111xBasedCacGt]): ...



import dataclasses
from dataclasses import dataclass


@dataclass
class DecodedMQTTTopic:
    envelope_type: str = ""
    src: str = ""
    dst: str = ""
    message_type: str = ""
    remainder: list[str] = dataclasses.field(default_factory=list)

    def __str__(self) -> str:
        return f"DecodedMQTTTopic {self.envelope_type}/{self.src}/{self.message_type} remainder:{self.remainder}"


class MQTTTopic:
    """Valid Gridworks MQTT topic is one of:

     - ENVELOPE_TYPE/SRC/to/DST/MESSAGE_TYPE, for example:
       'gw/hw1-isone-ma-boston-scada/to/a/report-event',
       meaning a message using envelope format 'gw', from
       'hw1-isone-ma-boston-scada', to the ATN ('a') with message type
       'report-event'. Note that SRC is the GNode Alias (a long name) whereas
       DST is the Spaceheat Name.

     - ENVELOPE_TYPE/SRC/to/DST, for example:
       'gw/hw1-isone-ma-boston-scada/to/a',
       meaning a message using envelope format 'gw', from
       'hw1-isone-ma-boston-scada' to the ATN ('a'). Message type must be
       inferred the contents of the envelope.

     - ENVELOPE_TYPE/SRC, for example:
       'gw/hw1-isone-ma-boston-scada',
       meaning a message using envelope format 'gw', from
       'hw1-isone-ma-boston-scada' Destination and message type must be inferred
       the contents of the envelope.

     - ENVELOPE_TYPE, for example: 'gw', meaning a message using envelope format
       'gw'. Source, destination and message type must be inferred the contents
       of the envelope.

     - ENVELOPE_TYPE/SRC/to/DST/MESSAGE_TYPE/OTHER/STUFF, for example:
       'gw/hw1-isone-ma-boston-scada/to/a/report-event/bla/bla',
       meaning a message using envelope format 'gw', from
       'hw1-isone-ma-boston-scada', to the ATN ('a') with message type
       'report-event', with extra components OTHER and STUFF that may be used by
       code down-stream code. OTHER and STUFF will available in the remainer
       attribute of the decoded topic. In this example the remainder would be
       ['bla', 'bla']

     - Note that if the third level down is not "to", everthing from that third
       level down is put in the remainder, so that, for example
       'gw/hw1-isone-ma-boston-scada/X/a/report-event/bla/bla', would be
       interpreted as evenlope 'gw', source 'hw1-isone-ma-boston-scada' and
       remainder ['X', 'a', 'report-event', 'bla', 'bla'].

     - If the third level is "to" but there is no fourth level,
       destination and message type will be empty but "to" not be placed in the
       in the remainder.

    The only valid component separator is '/'.

    '.' is not a valid character inside the mqtt topic. It must be replaced with
    '-'.

    It is an error for the message type contained inside the envelope to differ
    from one specified in the topic.
    """

    DOT = "."
    DOT_REPLACEMENT = "-"

    @classmethod
    def encode(cls, envelope_type: str, src: str, dst: str, message_type: str) -> str:
        return f"{envelope_type}/{src}/to/{dst}/{message_type}".replace(
            cls.DOT, cls.DOT_REPLACEMENT
        )

    @classmethod
    def encode_subscription(cls, envelope_type: str, src: str, dst: str) -> str:
        return cls.encode(envelope_type, src, dst, "#")

    @classmethod
    def decode(cls, topic: str) -> DecodedMQTTTopic:
        if not topic:
            raise ValueError("ERROR. Topic must have at least one character")
        topic = str(topic).replace(cls.DOT_REPLACEMENT, cls.DOT)
        split = topic.split("/")
        envelope_type = split[0]
        # for example: 'gw'
        remainder: list[str]
        if len(split) == 1:
            src = ""
            dst = ""
            message_type = ""
            remainder = []
        # for example: 'gw/hw1-isone-ma-boston-scada'
        elif len(split) == 2:
            src = split[1]
            dst = ""
            message_type = ""
            remainder = []
        # len > 2
        elif split[2] != "to":
            src = split[1]
            dst = ""
            message_type = ""
            remainder = split[2:]
        elif len(split) == 3:
            src = split[1]
            dst = ""
            message_type = ""
            remainder = []
        # for example: 'gw/hw1-isone-ma-boston-scada/to/a'
        elif len(split) == 4:
            src = split[1]
            dst = split[3]
            message_type = ""
            remainder = []
        # for example: 'gw/hw1-isone-ma-boston-scada/to/a/report-event'
        #          or: 'gw/hw1-isone-ma-boston-scada/to/a/report-event/x'
        else:
            src = split[1]
            dst = split[3]
            message_type = split[4]
            remainder = split[5:]
        return DecodedMQTTTopic(
            envelope_type=envelope_type,
            src=src,
            dst=dst,
            message_type=message_type,
            remainder=remainder,
        )



"""Type egauge.register.config, version 000"""

from typing import Literal

from pydantic import BaseModel, Field


class EgaugeRegisterConfig(BaseModel):
    Address: int = Field(
        title="Address",
        description=(
            "EGauge's modbus holding address. Note that the EGauge modbus map for holding address "
            "100 will be 30100 - the '+30000' indicates it is a holding address. We use the 4-digit "
            "address after the '3'."
        ),
    )
    Name: str = Field(
        title="Name",
        description=(
            "The name assigned in the EGauge's modbus map. This is configured by the user (see "
            "URL)"
            "[More info](https://docs.google.com/document/d/1VeAt-V_AVqqiB0EVf-4JL_k_hVOsgbqldeAPVNwG1yI/edit#heading=h.7ct5hku166ut)"
        ),
    )
    Description: str = Field(
        title="Description",
        description="Again, assigned by the EGauge modbus map. Is usually 'change in value'",
    )
    Type: str = Field(
        title="Type",
        description=(
            "EGauge's numerical data type. Typically our power measurements are f32 ( 32-bit "
            "floating-point number). The serial number & firmware are t16 (which work to treat "
            "as 16-bit unsigned integer) and timestamps are u32 (32-bit unsigned integer)."
        ),
    )
    Denominator: int = Field(
        title="Denominator",
        description=(
            "Some of the modbus registers divide by 3.60E+06 (cumulative energy registers typically). "
            "For the power, current, voltage and phase angle the denominator is 1."
        ),
    )
    Unit: str = Field(
        title="Unit",
        description="The EGauge unit - typically A, Hz, or W.",
    )
    TypeName: Literal["egauge.register.config"] = "egauge.register.config"
    Version: str = "000"



"""Type single.reading, version 000"""

from typing import Literal

from pydantic import BaseModel, ConfigDict, StrictInt

from gwproto.property_format import (
    SpaceheatName,
    UTCMilliseconds,
)


class SingleReading(BaseModel):
    ChannelName: SpaceheatName
    Value: StrictInt
    ScadaReadTimeUnixMs: UTCMilliseconds
    TypeName: Literal["single.reading"] = "single.reading"
    Version: str = "000"

    model_config = ConfigDict(use_enum_values=True)



# ruff: noqa: ANN401, RUF100,


import gwproto.named_types.cacs
import gwproto.named_types.components
from gwproto.decoders import CacDecoder, ComponentDecoder

__all__ = [
    "default_cac_decoder",
    "default_component_decoder",
]

default_cac_decoder = CacDecoder(
    model_name="DefaultCacDecoder",
    modules=[gwproto.named_types.cacs],
)

default_component_decoder = ComponentDecoder(
    model_name="DefaultComponentDecoder",
    modules=[gwproto.named_types.components],
)



"""Type send.snap, version 000"""

from typing import Literal

from pydantic import BaseModel

from gwproto.property_format import (
    LeftRightDotStr,
)


class SendSnap(BaseModel):
    FromGNodeAlias: LeftRightDotStr
    TypeName: Literal["send.snap"] = "send.snap"
    Version: str = "000"



from typing import Literal

from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.web_server_gt import WebServerGt


class WebServerComponentGt(ComponentGt):
    WebServer: WebServerGt
    TypeName: Literal["web.server.component.gt"] = "web.server.component.gt"



from gwproto.named_types.hubitat_component_gt import HubitatRESTResolutionSettings
from gwproto.named_types.hubitat_gt import HubitatGt
from gwproto.named_types.hubitat_poller_gt import HubitatPollerGt, MakerAPIAttributeGt
from gwproto.named_types.hubitat_tank_gt import (
    FibaroTempSensorSettings,
    FibaroTempSensorSettingsGt,
    HubitatTankSettingsGt,
)
from gwproto.named_types.rest_poller_gt import (
    AioHttpClientTimeout,
    RequestArgs,
    RESTPollerSettings,
    SessionArgs,
    URLArgs,
    URLConfig,
)
from gwproto.named_types.web_server_gt import WebServerGt
from gwproto.type_helpers.cacs_by_make_model import CACS_BY_MAKE_MODEL

__all__ = [
    "CACS_BY_MAKE_MODEL",
    "AioHttpClientTimeout",
    "FibaroTempSensorSettings",
    "FibaroTempSensorSettingsGt",
    "HubitatGt",
    "HubitatPollerGt",
    "HubitatRESTResolutionSettings",
    "HubitatTankSettingsGt",
    "MakerAPIAttributeGt",
    "RESTPollerSettings",
    "RequestArgs",
    "SessionArgs",
    "URLArgs",
    "URLConfig",
    "WebServerGt",
]



"""Type data.channel.gt, version 001"""

from typing import Literal, Optional

from pydantic import BaseModel, ConfigDict, model_validator
from typing_extensions import Self

from gwproto.enums import TelemetryName
from gwproto.property_format import (
    LeftRightDotStr,
    SpaceheatName,
    UTCSeconds,
    UUID4Str,
)


class DataChannelGt(BaseModel):
    Name: SpaceheatName
    DisplayName: str
    AboutNodeName: SpaceheatName
    CapturedByNodeName: SpaceheatName
    TelemetryName: TelemetryName
    TerminalAssetAlias: LeftRightDotStr
    InPowerMetering: Optional[bool] = None
    StartS: Optional[UTCSeconds] = None
    Id: UUID4Str
    TypeName: Literal["data.channel.gt"] = "data.channel.gt"
    Version: str = "001"

    @model_validator(mode="after")
    def check_axiom_1(self) -> Self:
        """
        Axiom 1: Power Metering.
        If InPowerMetering is true then the TelemetryName must be PowerW
        """
        if self.InPowerMetering and self.TelemetryName != TelemetryName.PowerW:
            raise ValueError(
                "Axiom 1 violated! If InPowerMetering is true then"
                f"the TelemetryName must be PowerW. Got  {self.TelemetryName}"
            )
        return self

    model_config = ConfigDict(use_enum_values=True)



"""Type ads.channel.config, version 000"""

from typing import Literal, Optional

from pydantic import ConfigDict, PositiveInt

from gwproto.enums import MakeModel, ThermistorDataMethod
from gwproto.named_types.channel_config import ChannelConfig
from gwproto.property_format import (
    SpaceheatName,
)


class AdsChannelConfig(ChannelConfig):
    ChannelName: SpaceheatName
    TerminalBlockIdx: PositiveInt
    ThermistorMakeModel: MakeModel
    DataProcessingMethod: Optional[ThermistorDataMethod] = None
    DataProcessingDescription: Optional[str] = None
    TypeName: Literal["ads.channel.config"] = "ads.channel.config"
    Version: str = "000"

    model_config = ConfigDict(extra="allow", use_enum_values=True)



"""Type ticklist.hall, version 101"""

from typing import Literal, Optional

from pydantic import BaseModel, StrictInt, model_validator  # Count:true
from typing_extensions import Self


class TicklistHall(BaseModel):
    HwUid: str
    FirstTickTimestampNanoSecond: Optional[StrictInt] = None
    RelativeMicrosecondList: list[StrictInt]
    PicoBeforePostTimestampNanoSecond: StrictInt
    TypeName: Literal["ticklist.hall"] = "ticklist.hall"
    Version: str = "101"

    @model_validator(mode="after")
    def check_axiom_1(self) -> Self:
        """
        Axiom 1: FirstTickTimestampNanoSecond is none iff RelativeMicrosecondList has length 0.

        """
        if (
            self.FirstTickTimestampNanoSecond is None
            and len(self.RelativeMicrosecondList) > 0
        ):
            raise ValueError(
                "FirstTickTimestampNanoSecond is None but  RelativeMicrosecondList has nonzero length!"
            )
        if self.FirstTickTimestampNanoSecond and len(self.RelativeMicrosecondList) == 0:
            raise ValueError(
                "FirstTickTimestampNanoSecond exists but  RelativeMicrosecondList has no elements!"
            )
        return self



from typing import Literal

from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.hubitat_poller_gt import HubitatPollerGt


class HubitatPollerComponentGt(ComponentGt):
    Poller: HubitatPollerGt
    TypeName: Literal["hubitat.poller.component.gt"] = "hubitat.poller.component.gt"



"""Type tank.module.params, version 100"""

from typing import Literal, Optional

from pydantic import BaseModel, PositiveInt, model_validator
from typing_extensions import Self

from gwproto.property_format import (
    SpaceheatName,
)


class TankModuleParams(BaseModel):
    """
    Parameters for a  GRIDWORKS__TANKMODULE2 device or a GRIDWORKS__TANKMODULE3 device
    """

    HwUid: str
    ActorNodeName: SpaceheatName
    PicoAB: Optional[str] = None
    CapturePeriodS: PositiveInt
    Samples: PositiveInt
    NumSampleAverages: PositiveInt
    AsyncCaptureDeltaMicroVolts: PositiveInt
    CaptureOffsetS: Optional[float] = None
    TypeName: Literal["tank.module.params"] = "tank.module.params"
    Version: str = "110"

    @model_validator(mode="after")
    def check_pico_a_b(self) -> Self:
        """
        Axiom 1: "If PicoAB exists it must be a or b"
        """
        if self.PicoAB and self.PicoAB not in ["a", "b"]:
            raise ValueError(
                f"Axiom 1: If PicoAB exists it must be a or b, not {self.PicoAB}"
            )

        return self



import re
from typing import Literal, Optional

from pydantic import field_validator

from gwproto.enums import GpmFromHzMethod, HzCalcMethod, MakeModel
from gwproto.named_types.component_gt import ComponentGt
from gwproto.property_format import SpaceheatName


class PicoFlowModuleComponentGt(ComponentGt):
    Enabled: bool
    SerialNumber: str
    FlowNodeName: SpaceheatName
    FlowMeterType: MakeModel = MakeModel.SAIER__SENHZG1WA
    HzCalcMethod: HzCalcMethod
    GpmFromHzMethod: GpmFromHzMethod
    ConstantGallonsPerTick: float
    SendHz: bool = True
    SendGallons: bool = False
    SendTickLists: bool = False
    NoFlowMs: int
    AsyncCaptureThresholdGpmTimes100: int
    PublishEmptyTicklistAfterS: Optional[int] = None  # Hall Params
    PublishAnyTicklistAfterS: Optional[int] = None  # Reed Params
    PublishTicklistPeriodS: Optional[int] = None  # Required for Hall Params
    PublishTicklistLength: Optional[int] = None  # required for Reed Params
    ExpAlpha: Optional[float] = None
    CutoffFrequency: Optional[float] = None
    TypeName: Literal["pico.flow.module.component.gt"] = "pico.flow.module.component.gt"
    Version: str = "000"

    @field_validator("HwUid")
    @classmethod
    def check_hw_uid(cls, v: str) -> str:
        """
        Axiom 1: HwUid is of the form 'pico_xxxxxx' where xxxxxx
        are lowercase hex (the last digits of its pico W hw id)
        """
        pattern = r"^pico_[0-9a-f]{6}$"
        if not bool(re.match(pattern, v)):
            raise ValueError("HwUid should be the pico hwuid, eg pico_60e352")
        return v



from pydantic import ConfigDict

from gwproto.data_classes.sh_node import ShNode
from gwproto.named_types import DataChannelGt


class DataChannel(DataChannelGt):
    about_node: ShNode
    captured_by_node: ShNode

    model_config = ConfigDict(arbitrary_types_allowed=True, use_enum_values=True)

    def __hash__(self) -> int:
        return hash(self.Id)

    def to_gt(self) -> DataChannelGt:
        # Copy the current instance excluding the extra fields
        return DataChannelGt(
            **self.model_dump(exclude={"about_node", "captured_by_node"})
        )



"""Type machine.states, version 000"""

from typing import Literal

from pydantic import BaseModel, model_validator
from typing_extensions import Self

from gwproto.property_format import (
    HandleName,
    LeftRightDotStr,
    UTCMilliseconds,
)


class MachineStates(BaseModel):
    """ """

    MachineHandle: HandleName
    StateEnum: LeftRightDotStr
    StateList: list[str]
    UnixMsList: list[UTCMilliseconds]
    TypeName: Literal["machine.states"] = "machine.states"
    Version: str = "000"

    @model_validator(mode="after")
    def check_axiom_1(self) -> Self:
        """
        Axiom 1: list Length Consistency.
        StateList and UnixMsList must have the same length
        """
        # Implement check for axiom 1"
        return self

    @model_validator(mode="after")
    def check_axiom_2(self) -> Self:
        """
        Axiom 2: If StateEnum is a recognized GridWorks enum, then the StateList elements are all values of that enum..

        """
        # Implement check for axiom 2"
        return self



"""Type resistive.heater.component.gt, version 000"""

from typing import Literal, Optional

from gwproto.named_types import ComponentGt


class ResistiveHeaterComponentGt(ComponentGt):
    TestedMaxHotMilliOhms: Optional[int] = None
    TestedMaxColdMilliOhms: Optional[int] = None
    TypeName: Literal["resistive.heater.component.gt"] = "resistive.heater.component.gt"



from enum import auto

from gw.enums import GwStrEnum


class KindOfParam(GwStrEnum):
    """
    Supports tracking and updating various key parameters for Spaceheat SCADA and AtomicTNodes.
    This is meant in particular for by-hand updates meant to be made and later reviewed by humans,
    as opposed to a careful and precise database update process.
    Values:
      - Other: A kind of parameter other than others articulated in the enumeration. This
        can of course change depending on the version of the enum. For version 000, it means
        any kind of parameter other than those in a .env file or in the hardware layout of a
        SCADA.
      - HardwareLayout: A key parameter embedded with the hardware layout object of a
        spaceheat SCADA. See for example https://github.com/thegridelectric/gw-scada-spaceheat-python.
      - DotEnv: A key parameter from the .env file of a spaceheat SCADA, or a spaceheat
        AtomicTNode.

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheatkindofparam)
    """

    Other = auto()
    HardwareLayout = auto()
    DotEnv = auto()

    @classmethod
    def default(cls) -> "KindOfParam":
        return cls.Other

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "spaceheat.kind.of.param"

    @classmethod
    def enum_version(cls) -> str:
        return "000"



from enum import auto

from gw.enums import GwStrEnum


class StoreFlowRelay(GwStrEnum):
    """
    Used for a double-throw relay that can toggle between a thermal store heating up (flow is
    in the charging direction) or cooling down (flow is in the discharging direction). Events
    in the StoreFlowDirection finite state machine
    Values:
      - DischargingStore: Thermal store is discharging (and heating the building)
      - ChargingStore: Thermal store is charging from an external source (like a heat
        pump or oil boiler).

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#storeflowrelay)
    """

    DischargingStore = auto()
    ChargingStore = auto()

    @classmethod
    def default(cls) -> "StoreFlowRelay":
        return cls.DischargingStore

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "store.flow.relay"

    @classmethod
    def enum_version(cls) -> str:
        return "000"



# Literal Enum:
#  - no additional values can be added over time.
#  - Sent as-is, not in hex symbol
from enum import auto

from gw.enums import GwStrEnum


class ChangeRelayPin(GwStrEnum):
    """
    Clarifies the event request sent to an internal multiplexing actor regarding a single relay
    on a relay board (energize or de-energize).
    """

    DeEnergize = auto()
    Energize = auto()

    @classmethod
    def values(cls) -> list[str]:
        """
        Returns enum choices
        """
        return [elt.value for elt in cls]

    @classmethod
    def default(cls) -> "ChangeRelayPin":
        return cls.DeEnergize

    @classmethod
    def enum_name(cls) -> str:
        return "change.relay.pin"



"""Type synth.channel.gt, version 000"""

from typing import Literal

from pydantic import BaseModel, PositiveInt

from gwproto.enums import TelemetryName
from gwproto.property_format import (
    LeftRightDotStr,
    SpaceheatName,
    UUID4Str,
)


class SynthChannelGt(BaseModel):
    """
    Synthesized Channel.

    A descriptor for time-series data channel synthesized from multiple sources (instead of
    the raw telemetry captured by data.channel.gt
    """

    Id: UUID4Str
    Name: SpaceheatName
    CreatedByNodeName: SpaceheatName
    TelemetryName: TelemetryName
    TerminalAssetAlias: LeftRightDotStr
    Strategy: str
    DisplayName: str
    SyncReportMinutes: PositiveInt
    TypeName: Literal["synth.channel.gt"] = "synth.channel.gt"
    Version: str = "000"



import json


class StreamlinedSerializerMixin:
    @property
    def streamlined_serialize(self) -> str:
        return json.dumps(
            {key: value for key, value in self.__dict__.items() if value is not None}
        )



import re

snake_add_underscore_to_camel_pattern = re.compile(r"(?<!^)(?=[A-Z])")


def camel_to_snake(name: str) -> str:
    return snake_add_underscore_to_camel_pattern.sub("_", name).lower()


def snake_to_camel(word: str) -> str:
    return "".join(x.capitalize() or "_" for x in word.split("_"))


def rld_alias(alias: str) -> str:
    return ".".join(reversed(alias.split(".")))



from typing import Literal, Optional

import yarl
from pydantic_extra_types.mac_address import MacAddress

from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.hubitat_gt import HubitatGt
from gwproto.named_types.rest_poller_gt import URLConfig


class HubitatComponentGt(ComponentGt):
    Hubitat: HubitatGt
    TypeName: Literal["hubitat.component.gt"] = "hubitat.component.gt"

    def url_config(self) -> URLConfig:
        return self.Hubitat.url_config()

    def maker_api_url_config(self) -> URLConfig:
        return self.Hubitat.maker_api_url_config()

    def urls(self) -> dict[str, Optional[yarl.URL]]:
        return self.Hubitat.urls()

    def refresh_url_config(self, device_id: int) -> URLConfig:
        return self.Hubitat.refresh_url_config(device_id)

    def refresh_url(self, device_id: int) -> yarl.URL:
        return self.Hubitat.refresh_url(device_id)

    @classmethod
    def make_stub(cls, component_id: str) -> "HubitatComponentGt":
        return HubitatComponentGt(
            ComponentId=component_id,
            ComponentAttributeClassId="00000000-0000-4000-8000-000000000000",
            Hubitat=HubitatGt(
                Host="",
                MakerApiId=-1,
                AccessToken="",
                MacAddress=MacAddress("00:00:00:00:00:00"),
            ),
            ConfigList=[],
        )


class HubitatRESTResolutionSettings:
    component_gt: HubitatComponentGt
    maker_api_url_config: URLConfig

    def __init__(self, component_gt: HubitatComponentGt) -> None:
        self.component_gt = component_gt
        self.maker_api_url_config = self.component_gt.maker_api_url_config()



from gwproto.data_classes.components.component import Component
from gwproto.named_types.component_attribute_class_gt import ComponentAttributeClassGt
from gwproto.named_types.rest_poller_component_gt import RESTPollerComponentGt
from gwproto.named_types.rest_poller_gt import RESTPollerSettings


class RESTPollerComponent(Component[RESTPollerComponentGt, ComponentAttributeClassGt]):
    @property
    def rest(self) -> RESTPollerSettings:
        return self.gt.Rest



from enum import auto

from gw.enums import GwStrEnum


class TelemetryName(GwStrEnum):
    """
    Specifies the name of sensed data reported by a Spaceheat SCADA
    Values:
      - Unknown: Default Value - unknown telemetry name.
      - PowerW: Power in Watts.
      - RelayState: The Telemetry reading belongs to [1 ('Energized'), 0 ('DeEnergized')]
        (relay.energization.state enum).
      - WaterTempCTimes1000: Water temperature, in Degrees Celcius multiplied by 1000.
        Example: 43200 means 43.2 deg Celcius.
      - WaterTempFTimes1000: Water temperature, in Degrees F multiplied by 1000. Example:
        142100 means 142.1 deg Fahrenheit.
      - GpmTimes100: Gallons Per Minute multiplied by 100. Example: 433 means 4.33 gallons
        per minute.
      - CurrentRmsMicroAmps: Current measurement in Root Mean Square MicroAmps.
      - GallonsTimes100: Gallons multipled by 100. This is useful for flow meters that
        report cumulative gallons as their raw output. Example: 55300 means 55.3 gallons.
      - VoltageRmsMilliVolts: Voltage in Root Mean Square MilliVolts.
      - MilliWattHours: Energy in MilliWattHours.
      - MicroHz: Frequency in MicroHz. Example: 59,965,332 means 59.965332 Hz.
      - AirTempCTimes1000: Air temperature, in Degrees Celsius multiplied by 1000. Example:
        6234 means 6.234 deg Celcius.
      - AirTempFTimes1000: Air temperature, in Degrees F multiplied by 1000. Example:
        69329 means 69.329 deg Fahrenheit.
      - ThermostatState: Thermostat State: 0 means idle, 1 means heating, 2 means pending
        heat
      - MicroVolts: Microvolts RMS
      - VoltsTimesTen
      - WattHours
      - StorageLayer

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheattelemetryname)
      - [More Info](https://gridworks-protocol.readthedocs.io/en/latest/telemetry-name.html)
    """

    Unknown = auto()
    PowerW = auto()
    RelayState = auto()
    WaterTempCTimes1000 = auto()
    WaterTempFTimes1000 = auto()
    GpmTimes100 = auto()
    CurrentRmsMicroAmps = auto()
    GallonsTimes100 = auto()
    VoltageRmsMilliVolts = auto()
    MilliWattHours = auto()
    MicroHz = auto()
    AirTempCTimes1000 = auto()
    AirTempFTimes1000 = auto()
    ThermostatState = auto()
    MicroVolts = auto()
    VoltsTimesTen = auto()
    WattHours = auto()
    StorageLayer = auto()
    PercentKeep = auto()
    CelsiusTimes100 = auto()
    VoltsTimes100 = auto()
    HzTimes100 = auto()

    @classmethod
    def default(cls) -> "TelemetryName":
        return cls.Unknown

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "spaceheat.telemetry.name"

    @classmethod
    def enum_version(cls) -> str:
        return "006"



from typing import Any, Optional

import yarl

from gwproto.data_classes.components import HubitatComponent
from gwproto.data_classes.components.component import Component
from gwproto.data_classes.resolver import ComponentResolver
from gwproto.data_classes.sh_node import ShNode
from gwproto.named_types.component_attribute_class_gt import ComponentAttributeClassGt
from gwproto.named_types.hubitat_component_gt import HubitatComponentGt
from gwproto.named_types.hubitat_poller_component_gt import HubitatPollerComponentGt
from gwproto.named_types.rest_poller_gt import RequestArgs, RESTPollerSettings


class HubitatPollerComponent(
    Component[HubitatPollerComponentGt, ComponentAttributeClassGt], ComponentResolver
):
    hubitat_gt: HubitatComponentGt
    _rest: Optional[RESTPollerSettings] = None

    def __init__(
        self, gt: HubitatPollerComponentGt, cac: ComponentAttributeClassGt
    ) -> None:
        super().__init__(gt, cac)
        self.hubitat_gt = HubitatComponentGt.make_stub(gt.Poller.hubitat_component_id)

    @property
    def rest(self) -> RESTPollerSettings:
        if self._rest is None:
            raise ValueError(
                f"ERROR. resolve_rest() has not yet been called for {self}."
            )
        return self._rest

    def resolve(
        self,
        node_name: str,
        _nodes: dict[str, ShNode],
        components: dict[str, Component[Any, Any]],
    ) -> None:
        if self._rest is not None:
            raise ValueError(
                f"resolve() must be called exactly once. "
                f"Rest settings already exist for {self}."
            )

        # replace proxy hubitat component, which only had component id.
        # with the actual hubitat component containing data.
        hubitat_component = components.get(self.hubitat_gt.ComponentId, None)
        if hubitat_component is None or not isinstance(
            hubitat_component, HubitatComponent
        ):
            raise ValueError(
                f"ERROR. Component for {self.hubitat_gt.ComponentId} "
                f"has type <{type(hubitat_component)}>. Expected <HubitatComponent>"
            )
        self.hubitat_gt = hubitat_component.gt

        # Constuct url config on top of maker api url url config
        self._rest = RESTPollerSettings(
            request=RequestArgs(
                url=self.hubitat_gt.refresh_url_config(self.gt.Poller.device_id)
            ),
            poll_period_seconds=self.gt.Poller.poll_period_seconds,
        )

        # register attributes which accept web posts
        if (
            self.gt.Poller.web_listen_enabled
            and hubitat_component.gt.Hubitat.WebListenEnabled
        ):
            for attribute in self.gt.Poller.attributes:
                if attribute.web_listen_enabled:
                    hubitat_component.add_web_listener(node_name)

    def urls(self) -> dict[str, Optional[yarl.URL]]:
        urls = self.hubitat_gt.urls()
        for attribute in self.gt.Poller.attributes:
            urls[attribute.node_name] = self.rest.url
        return urls



"""Type component.gt, version 001"""

from collections.abc import Sequence
from typing import Optional

from pydantic import BaseModel, field_validator

from gwproto.named_types.channel_config import ChannelConfig
from gwproto.property_format import UUID4Str


class ComponentGt(BaseModel):
    ComponentId: UUID4Str
    ComponentAttributeClassId: UUID4Str
    ConfigList: Sequence[ChannelConfig]
    DisplayName: Optional[str] = None
    HwUid: Optional[str] = None
    TypeName: str = "component.gt"
    Version: str = "001"

    @field_validator("ConfigList")
    @classmethod
    def check_config_list(cls, v: Sequence[ChannelConfig]) -> Sequence[ChannelConfig]:
        """
        Axiom 1: Channel Name uniqueness. Data Channel names are
        unique in the config list
        """
        # Implement Axiom(s)
        return v



"""Old School enum (uses integers)"""

from enum import Enum


class RelayPinSet(Enum):
    """
    This refers to an ACTION taken by a driver (as opposed to a relay energization pin STATE
    read by the same driver). It is an old-school enum, where DeEnergized encodes 0 and Energized
    encodes 1.
    """

    DeEnergized = 0
    Energized = 1

    @classmethod
    def values(cls) -> list[int]:
        """
        Returns enum choices
        """
        return [elt.value for elt in cls]

    @classmethod
    def default(cls) -> "RelayPinSet":
        return cls.DeEnergized

    @classmethod
    def enum_name(cls) -> str:
        return "relay.pin.set"



from enum import auto

from gw.enums import GwStrEnum


class HeatcallSource(GwStrEnum):
    """
    Used for reflecting the state of a double-throw relay that can toggle between a failsafe
    for providing a 24V heat call to a zone controller (aka Wall Thermostat) and the SCADA providing
    that heat call
    Values:
      - WallThermostat
      - Scada

    For more information:
      - [ASLs](https://gridworks-type-registry.readthedocs.io/en/latest/)
      - [Global Authority](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#heatcallsource)
    """

    WallThermostat = auto()
    Scada = auto()

    @classmethod
    def default(cls) -> "HeatcallSource":
        return cls.WallThermostat

    @classmethod
    def values(cls) -> list[str]:
        return [elt.value for elt in cls]

    @classmethod
    def enum_name(cls) -> str:
        return "heatcall.source"

    @classmethod
    def enum_version(cls) -> str:
        return "000"



"""
GridWorks Enums used in gwproto, the Application Shared Language (ASL) used by SCADA
devices and AtomicTNodes to communicate with each other. These enums play a specific structural
role as semantic "glue" within ASLs.

Key attributes:
  - Enum values are translated into "GridWorks Type Enum Symbols" (GtEnumSymbols) when embedded
  in a serialized type sent as a message from one Application and/or Actor to another.
  - Each Enum has a unique name in the type registry (like spaceheat.telemetry.name), along
  with a version (like 001).
  - That name are interpretted locally in the SDK and do not necessarily carry the larger
  context of the unique type registry name (for example gwproto uses TelemetryName, since
  the `spaceheat` context goes without saying).
  - Each Value/Symbol pair also has a version. Value/Symbol pairs cannot be changed or removed.
  The only adjustments that can be made to an enum are adding more Value/Symbols. This is to
  support forwards- and backwards- compatability in GridWorks Types that use these enums.

If Enums are "glue", then GridWorks Types are the building blocks of SALs. Every SAL is comprised
of a set of shared GridWorks Types.

Application Shared Languages are an evolution of the concept of Application Programming Interfaces.
In a nutshell, an API can be viewed as a rather restricted version of an SAL, where only one application
has anything complex/interesting to say and, in general, the developers/owners of that application
have sole responsibility for managing the versioning and changing of that API. Note also that SALs
do not make any a priori assumption about the relationship (i.e. the default client/server for an API)
or the message delivery mechanism (i.e. via default GET/POST to RESTful URLs). For more information
on these ideas:
  - [GridWorks Enums](https://gridwork-type-registry.readthedocs.io/en/latest/types.html)
  - [GridWorks Types](https://gridwork-type-registry.readthedocs.io/en/latest/types.html)
  - [ASLs](https://gridwork-type-registry.readthedocs.io/en/latest/asls.html)
"""

from gwproto.enums.actor_class import ActorClass
from gwproto.enums.admin_event import AdminEvent
from gwproto.enums.admin_state import AdminState
from gwproto.enums.alert_priority import AlertPriority
from gwproto.enums.aquastat_control import AquastatControl
from gwproto.enums.change_aquastat_control import ChangeAquastatControl
from gwproto.enums.change_heat_pump_control import ChangeHeatPumpControl
from gwproto.enums.change_heatcall_source import ChangeHeatcallSource
from gwproto.enums.change_primary_pump_control import ChangePrimaryPumpControl
from gwproto.enums.change_relay_pin import ChangeRelayPin
from gwproto.enums.change_relay_state import ChangeRelayState
from gwproto.enums.change_store_flow_relay import ChangeStoreFlowRelay
from gwproto.enums.change_valve_state import ChangeValveState
from gwproto.enums.fsm_action_type import FsmActionType
from gwproto.enums.fsm_name import FsmName
from gwproto.enums.fsm_report_type import FsmReportType
from gwproto.enums.gpm_from_hz_method import GpmFromHzMethod
from gwproto.enums.heat_pump_control import HeatPumpControl
from gwproto.enums.heatcall_source import HeatcallSource
from gwproto.enums.hz_calc_method import HzCalcMethod
from gwproto.enums.kind_of_param import KindOfParam
from gwproto.enums.make_model import MakeModel
from gwproto.enums.primary_pump_control import PrimaryPumpControl
from gwproto.enums.relay_closed_or_open import RelayClosedOrOpen
from gwproto.enums.relay_energization_state import RelayEnergizationState
from gwproto.enums.relay_pin_set import RelayPinSet
from gwproto.enums.relay_wiring_config import RelayWiringConfig
from gwproto.enums.store_flow_relay import StoreFlowRelay
from gwproto.enums.strategy import Strategy
from gwproto.enums.telemetry_name import TelemetryName
from gwproto.enums.temp_calc_method import TempCalcMethod
from gwproto.enums.thermistor_data_method import ThermistorDataMethod
from gwproto.enums.unit import Unit

__all__ = [
    "ActorClass",  # [sh.actor.class.005](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#shactorclass)
    "AdminEvent",  # [admin.event.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#adminevent)
    "AdminState",  # [admin.state.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#adminstate)
    "AlertPriority",  # [alert.priority.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#alertpriority)
    "AquastatControl",  # [aquastat.control.state.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#aquastatcontrolstate)
    "ChangeAquastatControl",  # [change.aquastat.control.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changeaquastatcontrol)
    "ChangeHeatPumpControl",  # [change.heat.pump.control.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changeheatpumpcontrol)
    "ChangeHeatcallSource",  # [change.heatcall.source.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changeheatcallsource)
    "ChangePrimaryPumpControl",  # [change.primary.pump.control.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changeprimarypumpcontrol)
    "ChangeRelayPin",  # [change.relay.pin.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changerelaypin)
    "ChangeRelayState",  # [change.relay.state.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changerelaystate)
    "ChangeStoreFlowRelay",  # [change.store.flow.relay.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changestoreflowrelay)
    "ChangeValveState",  # [change.valve.state.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#changevalvestate)
    "FsmActionType",  # [sh.fsm.action.type.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#shfsmactiontype)
    "FsmName",  # [sh.fsm.name.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#shfsmname)
    "FsmReportType",  # [fsm.report.type.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#fsmreporttype)
    "GpmFromHzMethod",  # [gpm.from.hz.method.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#gpmfromhzmethod)
    "HeatPumpControl",  # [heat.pump.control.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#heatpumpcontrol)
    "HeatcallSource",  # [heatcall.source.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#heatcallsource)
    "HzCalcMethod",  # [hz.calc.method.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#hzcalcmethod)
    "KindOfParam",  # [spaceheat.kind.of.param.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheatkindofparam)
    "MakeModel",  # [spaceheat.make.model.004](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheatmakemodel)
    "PrimaryPumpControl",  # [primary.pump.control.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#primarypumpcontrol)
    "RelayClosedOrOpen",  # [relay.closed.or.open.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#relayclosedoropen)
    "RelayEnergizationState",  # [relay.energization.state.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#relayenergizationstate)
    "RelayPinSet",  # [relay.pin.set.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#relaypinset)
    "RelayWiringConfig",  # [relay.wiring.config.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#relaywiringconfig)
    "StoreFlowRelay",  # [store.flow.relay.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#storeflowrelay)
    "Strategy",  # [spaceheat.strategy.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheatstrategy)
    "TelemetryName",  # [spaceheat.telemetry.name.004](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheattelemetryname)
    "TempCalcMethod",  # [temp.calc.method.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#tempcalcmethod)
    "ThermistorDataMethod",  # [thermistor.data.method.000](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#thermistordatamethod)
    "Unit",  # [spaceheat.unit.001](https://gridworks-type-registry.readthedocs.io/en/latest/enums.html#spaceheatunit)
]



from gwproto.named_types.ads111x_based_component_gt import Ads111xBasedComponentGt
from gwproto.named_types.component_gt import ComponentGt
from gwproto.named_types.dfr_component_gt import DfrComponentGt
from gwproto.named_types.electric_meter_component_gt import ElectricMeterComponentGt
from gwproto.named_types.fibaro_smart_implant_component_gt import (
    FibaroSmartImplantComponentGt,
)
from gwproto.named_types.hubitat_component_gt import HubitatComponentGt
from gwproto.named_types.hubitat_poller_component_gt import HubitatPollerComponentGt
from gwproto.named_types.hubitat_tank_component_gt import HubitatTankComponentGt
from gwproto.named_types.i2c_multichannel_dt_relay_component_gt import (
    I2cMultichannelDtRelayComponentGt,
)
from gwproto.named_types.pico_btu_meter_component_gt import PicoBtuMeterComponentGt
from gwproto.named_types.pico_flow_module_component_gt import PicoFlowModuleComponentGt
from gwproto.named_types.pico_tank_module_component_gt import PicoTankModuleComponentGt
from gwproto.named_types.resistive_heater_component_gt import ResistiveHeaterComponentGt
from gwproto.named_types.rest_poller_component_gt import RESTPollerComponentGt
from gwproto.named_types.web_server_component_gt import WebServerComponentGt

__all__ = [
    "Ads111xBasedComponentGt",
    "ComponentGt",
    "DfrComponentGt",
    "ElectricMeterComponentGt",
    "FibaroSmartImplantComponentGt",
    "HubitatComponentGt",
    "HubitatPollerComponentGt",
    "HubitatTankComponentGt",
    "I2cMultichannelDtRelayComponentGt",
    "PicoBtuMeterComponentGt",
    "PicoFlowModuleComponentGt",
    "PicoTankModuleComponentGt",
    "RESTPollerComponentGt",
    "ResistiveHeaterComponentGt",
    "WebServerComponentGt",
]



# ruff: noqa: ANN401

import uuid
from typing import Any, Literal

from pydantic import BaseModel, Field

from gwproto.message import Message


class Ack(BaseModel):
    AckMessageID: str
    TypeName: Literal["gridworks.ack"] = "gridworks.ack"


class Ping(BaseModel):
    MessageId: str = Field(default_factory=lambda: str(uuid.uuid4()))
    TypeName: Literal["gridworks.ping"] = "gridworks.ping"


class PingMessage(Message[Ping]):
    def __init__(self, *, AckRequired: bool = True, **kwargs: Any) -> None:  # noqa: N803
        if "Payload" not in kwargs:
            kwargs["Payload"] = Ping()
        super().__init__(AckRequired=AckRequired, **kwargs)



# ruff: noqa: ANN401

from collections.abc import Mapping
from typing import (
    Any,
    Callable,
    Generic,
    Literal,
    Optional,
    TypeAlias,
    TypeVar,
    Union,
)

from pydantic import BaseModel

from gwproto.topic import MQTTTopic

EnumType = TypeVar("EnumType")


def as_enum(
    value: Any, enum_type: Callable[[Any], EnumType], default: Optional[EnumType] = None
) -> Optional[EnumType]:
    try:
        return enum_type(value)
    except ValueError:
        return default


class Header(BaseModel):
    Src: str
    Dst: str = ""
    MessageType: str
    MessageId: str = ""
    AckRequired: bool = False
    TypeName: Literal["gridworks.header"] = "gridworks.header"
    Version: str = "001"


# MyPy needs this because the class variable of Message is 'Header'.
# An alternative might be use snake case and set up pydantic to
# correct generate the aliases as expected.
HeaderT: TypeAlias = Header

PayloadT = TypeVar("PayloadT")

PAYLOAD_TYPE_FIELDS = ["TypeName", "type_alias", "TypeName", "type_name"]


def ensure_arg(arg_name: str, default_value: Any, kwargs_dict: dict[str, Any]) -> None:
    if arg_name not in kwargs_dict:
        payload = kwargs_dict.get("Payload")
        if payload is None or not hasattr(payload, arg_name):
            kwargs_dict[arg_name] = default_value


class Message(BaseModel, Generic[PayloadT]):
    Header: Header
    Payload: PayloadT
    TypeName: Literal["gw"] = "gw"

    def __init__(self, header: Optional[HeaderT] = None, **kwargs: Any) -> None:
        if header is None:
            header = self._header_from_kwargs(kwargs)
        super().__init__(Header=header, **kwargs)

    def message_type(self) -> str:
        return self.Header.MessageType

    def src(self) -> str:
        return self.Header.Src

    def dst(self) -> str:
        return self.Header.Dst

    @classmethod
    def type_name(cls) -> str:
        return str(Message.model_fields["TypeName"].default)

    def mqtt_topic(self) -> str:
        return MQTTTopic.encode(
            envelope_type=self.type_name(),
            src=self.src(),
            dst=self.dst(),
            message_type=self.message_type(),
        )

    @classmethod
    def _header_from_kwargs(cls, kwargs: dict[str, Any]) -> HeaderT:
        header_kwargs = {}
        if "Payload" in kwargs:
            payload = kwargs["Payload"]
            for header_field, payload_fields in [
                ("Src", ["Src"]),
                ("Dst", ["Dst"]),
                ("MessageId", ["MessageId"]),
                ("MessageType", PAYLOAD_TYPE_FIELDS),
                ("AckRequired", ["AckRequired"]),
            ]:
                val = kwargs.get(header_field)
                if val is None:
                    for payload_field in payload_fields:
                        if hasattr(payload, payload_field):
                            val = getattr(payload, payload_field)
                        elif isinstance(payload, Mapping) and payload_field in payload:
                            val = payload[payload_field]
                if val is not None:
                    header_kwargs[header_field] = val
        header: Optional[Union[Header, dict[str, Any]]] = kwargs.pop("Header", None)
        if isinstance(header, Header):
            header = header.model_copy(update=header_kwargs, deep=True)
        else:
            if header is not None:
                header_kwargs = dict(header, **header_kwargs)
            header = Header(**header_kwargs)
        return header


GRIDWORKS_ENVELOPE_TYPE = Message.type_name()



from gwproto.enums import MakeModel

CACS_BY_MAKE_MODEL: dict[str, str] = {
    MakeModel.EGAUGE__4030.value: "739a6e32-bb9c-43bc-a28d-fb61be665522",
    MakeModel.NCD__PR814SPST.value: "c6e736d8-8078-44f5-98bb-d72ca91dc773",
    MakeModel.ADAFRUIT__642.value: "43564cd2-0e78-41a2-8b67-ad80c02161e8",
    MakeModel.GRIDWORKS__TSNAP1.value: "432073b8-4d2b-4e36-9229-73893f33f846",
    MakeModel.GRIDWORKS__WATERTEMPHIGHPRECISION.value: "7937eb7e-24d5-4d52-990f-cca063484df9",
    MakeModel.GRIDWORKS__SIMPM1.value: "28897ac1-ea42-4633-96d3-196f63f5a951",
    MakeModel.SCHNEIDERELECTRIC__IEM3455.value: "6bcdc388-de10-40e6-979a-8d66bfcfe9ba",
    MakeModel.GRIDWORKS__SIMBOOL30AMPRELAY.value: "69f101fc-22e4-4caa-8103-50b8aeb66028",
    MakeModel.OPENENERGY__EMONPI.value: "357b9b4f-2550-4380-aa6b-d2cd9c7ba0f9",
    MakeModel.GRIDWORKS__SIMTSNAP1.value: "b9f7135e-07a9-42f8-b847-a9bb3ea3770a",
    MakeModel.ATLAS__EZFLO.value: "13d916dc-8764-4b16-b85d-b8ead3e2fc80",
    MakeModel.HUBITAT__C7__LAN1.value: "62528da5-b510-4ac2-82c1-3782842eae07",
    MakeModel.GRIDWORKS__TANK_MODULE_1.value: "60ac199d-679a-49f7-9142-8ca3e6428a5f",
    MakeModel.FIBARO__ANALOG_TEMP_SENSOR.value: "7ce0ce69-14c6-4cb7-a33f-2aeca91e0680",
    MakeModel.AMPHENOL__NTC_10K_THERMISTOR_MA100GG103BN.value: "2821c81d-054d-4003-9b07-2c295aef40f5",
    MakeModel.YHDC__SCT013100.value: "812761ba-6544-4796-9aad-e1c979f58734",
    MakeModel.MAGNELAB__SCT0300050.value: "cf312bd6-7ca5-403b-a61b-b2e817ea1e22",
    MakeModel.GRIDWORKS__MULTITEMP1.value: "b4db046e-500c-4ecd-86ed-a9dbaae039f9",
    MakeModel.KRIDA__EMR16I2CV3.value: "018d9ffb-89d1-4cc4-95c0-f170711b5ffa",
    MakeModel.OMEGA__FTB8007HWPT.value: "8cf6c726-e38a-4900-9cfe-ae6f053aafdf",
    MakeModel.ISTEC_4440.value: "62ed724c-ba62-4302-ae30-d52b20d42ad9",
    MakeModel.OMEGA__FTB8010HWPT.value: "d9f225f8-eeb5-4cb7-b314-5551b925ea27",
    MakeModel.BELIMO__BALLVALVE232VS.value: "a2236d8c-7c9b-403f-9c55-733c62971d09",
    MakeModel.BELIMO__DIVERTERB332L.value: "f3261ed0-3fb1-4def-b60b-246960bf85ef",
    MakeModel.TACO__0034EPLUS.value: "3880ba73-61e5-4b35-9df1-e154a03a3335",
    MakeModel.TACO__007E.value: "198ebac8-e0b9-4cee-ae91-2ee6db708491",
    MakeModel.ARMSTRONG__COMPASSH.value: "ff6863e1-d5f7-4066-8579-2768162321a6",
    MakeModel.HONEYWELL__T6ZWAVETHERMOSTAT.value: "03533a1f-3cb9-4a1f-8d57-690c0ad0475b",
    MakeModel.PRMFILTRATION__WM075.value: "61d5c12d-eeca-4835-9a11-e61167d82e0d",
    MakeModel.BELLGOSSETT__ECOCIRC20_18.value: "0d2ccc36-d2b8-405d-a257-3917111607c5",
    MakeModel.TEWA__TT0P10KC3T1051500.value: "20779dbb-0302-4c36-9d60-e1962857c2f3",
    MakeModel.EKM__HOTSPWM075HD.value: "e52cb571-913a-4614-90f4-5cc81f8e7fe5",
    MakeModel.GRIDWORKS__SIMMULTITEMP.value: "627ac482-24fe-46b2-ba8c-3d6f1e1ee069",
    MakeModel.GRIDWORKS__SIMTOTALIZER.value: "a88f8f4c-fe1e-4645-a7f4-249912131dc8",
    MakeModel.KRIDA__DOUBLEEMR16I2CV3.value: "29eab8b1-100f-4230-bb44-3a2fcba33cc3",
    MakeModel.GRIDWORKS__SIMDOUBLE16PINI2CRELAY.value: "44ff9d75-edb9-4249-afea-96822bfc55f3",
    MakeModel.GRIDWORKS__TANKMODULE2.value: "f88fbf89-5b74-46d6-84a3-8e7494d08435",
    MakeModel.GRIDWORKS__PICOFLOWHALL.value: "9d045a8d-12ee-43e6-8d7b-11c77b19de2f",
    MakeModel.GRIDWORKS__PICOFLOWREED.value: "aa4ad342-883a-4f89-bf86-9eb430aeb308",
    MakeModel.SAIER__SENHZG1WA.value: "58ecde10-b015-4711-9659-d5bfb7665393",
    MakeModel.DFROBOT__DFR0971_TIMES2.value: "69a0e36e-9c15-4d50-8a7b-ae63649ecc36",
    MakeModel.GRIDWORKS__TANKMODULE3.value: "cbe49338-5d14-47ff-b05e-22031876962e",
    MakeModel.GRIDWORKS__GW101.value: "d020608d-d5ed-4b12-9592-7135db34e4ba",
}



# Literal Enum:
#  - no additional values can be added over time.
#  - Sent as-is, not in hex symbol
from enum import auto

from gwproto.enums.relay_event_base import RelayEventBase


class ChangeHeatPumpControl(RelayEventBase):
    """
    Change control between a fallback analog system and SCADA
    """

    SwitchToTankAquastat = auto()
    SwitchToScada = auto()

    @classmethod
    def values(cls) -> list[str]:
        """
        Returns enum choices
        """
        return [elt.value for elt in cls]

    @classmethod
    def default(cls) -> "ChangeHeatPumpControl":
        return cls.SwitchToTankAquastat

    @classmethod
    def enum_name(cls) -> str:
        return "change.heat.pump.control"



from gwproto.named_types.ads111x_based_cac_gt import Ads111xBasedCacGt
from gwproto.named_types.electric_meter_cac_gt import ElectricMeterCacGt
from gwproto.named_types.resistive_heater_cac_gt import ResistiveHeaterCacGt

__all__ = [
    "Ads111xBasedCacGt",
    "ElectricMeterCacGt",
    "ResistiveHeaterCacGt",
]



import typing
from typing import Optional

import yarl
from pydantic import BaseModel, ConfigDict
from pydantic_extra_types.mac_address import MacAddress

from gwproto.named_types.rest_poller_gt import URLArgs, URLConfig


class URLConfigWithUrlArgs(URLConfig):
    """
    A URLConfig with non-None url_args
    """

    url_args: URLArgs


class URLConfigForMakerAPI(URLConfigWithUrlArgs):
    """
    A URLConfig with non-None url_args and url_path_args suitable for creating
    a MakerAPI url.
    """

    url_path_args: dict[str, str | int | float]


class HubitatGt(BaseModel):
    Host: str
    MakerApiId: int
    AccessToken: str
    MacAddress: MacAddress
    WebListenEnabled: bool = True
    model_config = ConfigDict(extra="allow", populate_by_name=True)

    @property
    def listen_path(self) -> str:
        return self.MacAddress.replace(":", "-")

    def listen_url(self, url: yarl.URL) -> yarl.URL:
        return url / self.listen_path

    def url_config(self) -> URLConfigWithUrlArgs:
        return URLConfigWithUrlArgs(
            url_args=URLArgs(
                scheme="http",
                host=self.Host,
            ),
        )

    def maker_api_url_config(self) -> URLConfigForMakerAPI:
        config = self.url_config()
        if config.url_args is None:
            raise ValueError(
                f"ERROR. URLConfig.url_args ({config.url_args})"
                " are insufficient to create MakerAPI url"
            )
        if config.url_args.query is None:
            config.url_args.query = []
        config.url_args.query.append(("access_token", self.AccessToken))
        config.url_path_format += "/apps/api/{app_id}"
        if config.url_path_args is None:
            config.url_path_args = {}
        config.url_path_args.update({"app_id": self.MakerApiId})
        return typing.cast(URLConfigForMakerAPI, config)

    def devices_url_config(self) -> URLConfig:
        config = self.maker_api_url_config()
        config.url_path_format += "/devices"
        return config

    def url_configs(self) -> dict[str, URLConfig]:
        return {
            "base": self.url_config(),
            "maker_api": self.maker_api_url_config(),
            "devices": self.devices_url_config(),
        }

    def urls(self) -> dict[str, Optional[yarl.URL]]:
        return {
            name: URLConfig.make_url(config)
            for name, config in self.url_configs().items()
        }

    def refresh_url_config(self, device_id: int) -> URLConfig:
        config = self.maker_api_url_config()
        config.url_path_format += "/devices/{device_id}/refresh"  # noqa: RUF027
        config.url_path_args["device_id"] = device_id
        return config

    def refresh_url(self, device_id: int) -> yarl.URL:
        url = URLConfig.make_url(self.refresh_url_config(device_id))
        if url is None:
            raise ValueError(
                f"ERROR. refreshed URL could not produce URL from <{self}> "
                f"and device_id {device_id}"
            )
        return url



"""DfrComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types import ComponentAttributeClassGt, DfrComponentGt


class DfrComponent(Component[DfrComponentGt, ComponentAttributeClassGt]): ...



"""PicoFlowModuleComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types import ComponentAttributeClassGt, PicoFlowModuleComponentGt


class PicoFlowModuleComponent(
    Component[PicoFlowModuleComponentGt, ComponentAttributeClassGt]
): ...



"""Type fsm.full.report, version 000"""

from typing import Literal

from pydantic import BaseModel, ConfigDict

from gwproto.named_types.fsm_atomic_report import FsmAtomicReport
from gwproto.property_format import (
    SpaceheatName,
    UUID4Str,
)


class FsmFullReport(BaseModel):
    FromName: SpaceheatName
    TriggerId: UUID4Str
    AtomicList: list[FsmAtomicReport]
    TypeName: Literal["fsm.full.report"] = "fsm.full.report"
    Version: str = "000"

    model_config = ConfigDict(extra="allow", use_enum_values=True)



"""Type report, version 002"""

from typing import Literal

from pydantic import BaseModel, PositiveInt, field_validator

from gwproto.named_types.channel_readings import ChannelReadings
from gwproto.named_types.fsm_full_report import FsmFullReport
from gwproto.named_types.machine_states import MachineStates
from gwproto.property_format import (
    LeftRightDotStr,
    UTCMilliseconds,
    UTCSeconds,
    UUID4Str,
)


class Report(BaseModel):
    """ """

    FromGNodeAlias: LeftRightDotStr
    FromGNodeInstanceId: UUID4Str
    AboutGNodeAlias: LeftRightDotStr
    SlotStartUnixS: UTCSeconds
    SlotDurationS: PositiveInt
    ChannelReadingList: list[ChannelReadings]
    StateList: list[MachineStates]
    FsmReportList: list[FsmFullReport]
    MessageCreatedMs: UTCMilliseconds
    Id: UUID4Str
    TypeName: Literal["report"] = "report"
    Version: str = "002"

    @field_validator("ChannelReadingList")
    @classmethod
    def _check_channel_reading_list(
        cls, v: list[ChannelReadings]
    ) -> list[ChannelReadings]:
        return v



"""I2cMultichannelDtRelayComponent definition"""

from gwproto.data_classes.components.component import Component
from gwproto.named_types import (
    ComponentAttributeClassGt,
    I2cMultichannelDtRelayComponentGt,
)


class I2cMultichannelDtRelayComponent(
    Component[I2cMultichannelDtRelayComponentGt, ComponentAttributeClassGt]
): ...



"""Type channel.readings, version 002"""

from typing import Literal

from pydantic import BaseModel, StrictInt, model_validator  # Count:true
from typing_extensions import Self

from gwproto.property_format import (
    SpaceheatName,
    UTCMilliseconds,
)


class ChannelReadings(BaseModel):
    """
    A list of timestamped readings (values) for a data channel. This is meant to be reported
    for non-local consumption (AtomicTNode, other) by a SCADA. Therefore, the data channel is
    referenced by its globally unique identifier. The receiver needs to reference this idea
    against a list of the data channels used by the SCADA for accurate parsing.
    """

    ChannelName: SpaceheatName
    ValueList: list[StrictInt]
    ScadaReadTimeUnixMsList: list[UTCMilliseconds]
    TypeName: Literal["channel.readings"] = "channel.readings"
    Version: str = "002"

    @model_validator(mode="after")
    def check_axiom_1(self) -> Self:
        """
        Axiom 1: ListLengthConsistency.
        ValueList and ScadaReadTimeUnixMsList must have the same length.
        """
        # Implement check for axiom 1"
        return self



from gwproto.data_classes.components.component import Component
from gwproto.named_types.component_attribute_class_gt import ComponentAttributeClassGt
from gwproto.named_types.fibaro_smart_implant_component_gt import (
    FibaroSmartImplantComponentGt,
)


class FibaroSmartImplantComponent(
    Component[FibaroSmartImplantComponentGt, ComponentAttributeClassGt]
): ...
